﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Random;

namespace HeuristicLab.Encodings.PermutationEncoding {
  /// <summary>
  /// An operator which creates a new random permutation of integer values.
  /// </summary>
  [Item("PreexistingPermutationCreator", "An operator which creates a new permutation of integer values from a set of given permutations.")]
  [StorableType("D6EC71C0-2AC1-461B-B684-13352AC88A8C")]
  public sealed class PreexistingPermutationCreator : InstrumentedOperator, IPermutationCreator, IPreexistingSolutionCreator, IStochasticOperator {
    public override bool CanChangeName {
      get { return false; }
    }

    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }
    public IValueLookupParameter<IntValue> LengthParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["Length"]; }
    }
    public ILookupParameter<Permutation> PermutationParameter {
      get { return (ILookupParameter<Permutation>)Parameters["Permutation"]; }
    }
    public IValueParameter<PermutationType> PermutationTypeParameter {
      get { return (IValueParameter<PermutationType>)Parameters["PermutationType"]; }
    }
    public IValueLookupParameter<ScopeList> PreexistingSolutionsParameter {
      get { return (IValueLookupParameter<ScopeList>)Parameters["PreexistingSolutions"]; }
    }
    public IValueLookupParameter<BoolValue> SampleFromPreexistingParameter {
      get { return (IValueLookupParameter<BoolValue>)Parameters["SampleFromPreexisting"]; }
    }

    public PermutationTypes PermutationType {
      get { return PermutationTypeParameter.Value.Value; }
      set { PermutationTypeParameter.Value.Value = value; }
    }

    [StorableConstructor]
    private PreexistingPermutationCreator(StorableConstructorFlag _) : base(_) { }
    private PreexistingPermutationCreator(PreexistingPermutationCreator original, Cloner cloner) : base(original, cloner) { }
    public PreexistingPermutationCreator()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator which should be used to sample from the existing permutations."));
      Parameters.Add(new ValueLookupParameter<IntValue>("Length", "The length of the new random permutation."));
      Parameters.Add(new LookupParameter<Permutation>("Permutation", "The new random permutation."));
      Parameters.Add(new ValueParameter<PermutationType>("PermutationType", "The type of the permutation.", new PermutationType(PermutationTypes.RelativeUndirected)));
      Parameters.Add(new ValueLookupParameter<ScopeList>("PreexistingSolutions", "The list of solutions that should be used to create a new solution.", new ScopeList())); 
      Parameters.Add(new ValueLookupParameter<BoolValue>("SampleFromPreexisting", "Whether the preexisting solutions should be used as a basis to sample from or whether they should be cloned.", new BoolValue(true)));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new PreexistingPermutationCreator(this, cloner);
    }

    public override IOperation InstrumentedApply() {
      var random = RandomParameter.ActualValue;
      var preexisting = (Permutation)PreexistingSolutionsParameter.ActualValue.SampleRandom(random).Variables[PermutationParameter.ActualName].Value;
      var sample = SampleFromPreexistingParameter.ActualValue.Value;
      if (sample) {
        var perm = new Permutation(PermutationTypeParameter.Value.Value, LengthParameter.Value.Value, random);
        PermutationParameter.ActualValue = perm.PermutationType == PermutationTypes.Absolute ? PartiallyMatchedCrossover.Apply(random, preexisting, perm)
                                                                                             : OrderCrossover.Apply(random, preexisting, perm);
      } else {
        PermutationParameter.ActualValue = (Permutation)preexisting.Clone();
      }
      return base.InstrumentedApply();
    }
  }
}
